package dtu.library.app;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import dtu.library.persistency.PersistencyLayer;

public class LibraryApp {

	private boolean adminLoggedIn = false;
	// private List<Medium> media = new ArrayList<Medium>();
	// private List<User> users = new ArrayList<User>();
	private DateServer dateServer = new DateServer();
	private MailService mailService;
	private PersistencyLayer pl = new PersistencyLayer();

	public List<Medium> getMedia() throws IOException {
		// return Collections.unmodifiableList(media);
		return pl.getMedia();

	}

	public boolean adminLoggedIn() {
		return adminLoggedIn;
	}

	public boolean adminLogin(String password) {
		return adminLoggedIn = "adminadmin".equals(password);
	}

	public void addMedium(Medium medium) throws OperationNotAllowedException, IOException {
		if (!adminLoggedIn)
			throw new OperationNotAllowedException("Add book");
		medium.setLibraryApp(this);
		// media.add(medium);
		pl.createMedium(medium);
	}

	public List<Medium> search(String string) throws IOException {
		ArrayList<Medium> result = new ArrayList<Medium>();
		for (Medium medium : getMedia()) {
			if (medium.getTitle().contains(string)
					|| medium.getAuthor().contains(string)
					|| medium.getSignature().contains(string)) {
				result.add(medium);
			}
		}
		return Collections.unmodifiableList(result);
	}

	public void adminLogoff() {
		adminLoggedIn = false;
	}

	public List<User> getUsers() throws IOException {
		// return Collections.unmodifiableList(users);
		return pl.getUsers();
	}

	public void register(User user) throws OperationNotAllowedException, IOException {
		if (!adminLoggedIn)
			throw new OperationNotAllowedException("Register user");
		user.setLibraryApp(this);
		// users.add(user);
		pl.createUser(user);
	}

	public User userByCprNumber(String string) throws IOException {
		// for (User user : users) {
		// if (string.equals(user.getCprNumber())) {
		// return user;
		// }
		// }
		// return null;
		return pl.readUser(string);
	}

	public Medium mediaBySignature(String signature) throws IOException {
		// for (Medium medium : media) {
		// if (signature.equals(medium.getSignature())) {
		// return medium;
		// }
		// }
		// return null;
		return pl.readMedium(signature);
	}

	public void deleteMedium(String signature)
			throws OperationNotAllowedException, IOException {
		if (!adminLoggedIn()) {
			throw new OperationNotAllowedException("Delete medium");
		}
		// Medium foundMedia = null;
		// for (Medium oneMedia : media) {
		// if (signature.equals(oneMedia.getSignature())) {
		// foundMedia = oneMedia;
		// }
		// }
		// media.remove(foundMedia);
		pl.deleteMedium(signature);
	}

	void setDateServer(DateServer dateServer) {
		this.dateServer = dateServer;

	}

	public Calendar getDate() {
		return dateServer.getDate();
	}

	public void setMailService(MailService ms) {
		mailService = ms;

	}

	public void sendEMailReminder() throws IOException {
		for (User user : getUsers()) {
			int count = 0;
			for (Medium medium : user.getBorrowedMedia()) {
				if (medium.isOverdue()) {
					count++;
				}
			}
			if (count > 0) {
				mailService.send(user.getEmail(), "Overdue book(s)",
						"You have " + count + " overdue book(s)");
			}
		}
	}

	public void unregister(String cpr) throws OperationNotAllowedException, IOException {
		if (!adminLoggedIn()) {
			throw new OperationNotAllowedException("Unregister user");
		}
		// User user = userByCprNumber(cpr);
		// users.remove(user);
		pl.deleteUser(cpr);
	}

	public void setPersistencyLayer(PersistencyLayer persistencyLayer) {
		this.pl = persistencyLayer;
	}

	public PersistencyLayer getPersistencyLayer() {
		return pl;
	}
}
