package dtu.library.app;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import dtu.library.persistency.PersistencyLayer;
import dtu.library.persistency.PersistentObject;


public abstract class Medium extends PersistentObject {
	private String signature;
	private String title;
	private String author;
	private Calendar borrowDate;

	public Medium(String signature, String title, String author) {
		this.signature = signature;
		this.title = title;
		this.author = author;
	}

	protected Medium() {
	};

	public void setSignature(String signature) {
		if (signature != null) return; // signature is read-only
		this.signature = signature;
	}

	public String getSignature() {
		return signature;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getTitle() {
		return title;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getAuthor() {
		return author;
	}

	void setBorrowDate(Calendar date) {
		borrowDate = date;
		try {
			libApp.getPersistencyLayer().updateMedium(this);
		} catch (IOException e) {
			throw new Error(e);
		}
	}

	public Calendar getBorrowDate() {
		return borrowDate;
	}

	public boolean isOverdue() {
		if (!isBorrowed()) {
			return false;
		}
		Calendar date = libApp.getDate();
		Calendar latestReturnDate = new GregorianCalendar();
		latestReturnDate.setTime(borrowDate.getTime());
		latestReturnDate.add(Calendar.DAY_OF_YEAR, getMaxDaysForLoan());
		return latestReturnDate.before(date);
	}

	protected abstract int getMaxDaysForLoan();

	public boolean isBorrowed() {
		return borrowDate != null;
	}

	void resetBorrowedStatus() throws IOException {
		borrowDate = null;
		libApp.getPersistencyLayer().updateMedium(this);
	}

	public abstract int getFine();

	@Override
	public void storeOn(PrintWriter out) {
		out.println(this.getClass().getName());
		out.println(signature);
		out.println(author);
		out.println(title);
		if (borrowDate == null) {
			out.println("null");
		} else {
			out.println(DateFormat.getDateInstance().format(borrowDate.getTime()));
		}
		out.println(); // Empty line to mark the end of the record for fast skipping
	}

	public String getKey() {
		return getSignature();
	}

	protected void readFromReader(PersistencyLayer persistentLayer, BufferedReader in)
			throws IOException {
		signature = in.readLine();
		author = in.readLine();
		title = in.readLine();
		String date = in.readLine();
		if (!"null".equals(date)) {
			borrowDate = new GregorianCalendar();
			try {
				borrowDate.setTime(DateFormat.getDateInstance().parse(date));
			} catch (ParseException e) {
				borrowDate = null;
			}
		}
		in.readLine(); // Read the end of record marker
	}
}
