package dtu.library.app;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dtu.library.persistency.PersistencyLayer;
import dtu.library.persistency.PersistentObject;

public class User extends PersistentObject {

	private String cprNumber;
	private String name;
	private String email;
	private Address address;
	private List<Medium> borrowedMedia = new ArrayList<Medium>();

	public User(String cprNumber, String name, String email, Address address) {
		this.cprNumber = cprNumber;
		this.name = name;
		this.email = email;
		this.address = address;
	}

	public User() {
	}

	public String getCprNumber() {
		return cprNumber;
	}

	public String getName() {
		return name;
	}

	public String getEmail() {
		return email;
	}

	public Address getAddress() {
		return address;
	}

	public List<Medium> getBorrowedMedia() {
		return Collections.unmodifiableList(borrowedMedia);
	}

	public void borrowMedium(Medium medium) throws BorrowException {
		if (medium == null)
			return;
		if (borrowedMedia.size() >= 10) {
			throw new TooManyBooksException();
		}
		for (Medium mdm : borrowedMedia) {
			if (mdm.isOverdue()) {
				throw new HasOverdueMedia();
			}
		}
		medium.setBorrowDate(libApp.getDate());
		borrowedMedia.add(medium);
		try {
			libApp.getPersistencyLayer().updateUser(this);
		} catch (IOException e) {
			throw new Error(e);
		}
	}

	public void returnMedia(Medium medium) throws IOException {
		medium.resetBorrowedStatus();
		borrowedMedia.remove(medium);
		libApp.getPersistencyLayer().updateUser(this);
	}

	public int getFine() {
		int fine = 0;
		for (Medium medium : borrowedMedia) {
			if (medium.isOverdue()) {
				fine += medium.getFine();
			}
		}
		return fine;
	}

	@Override
	public void storeOn(PrintWriter out) {
		out.println(this.getClass().getName());
		out.println(cprNumber);
		out.println(name);
		out.println(email);
		address.storeOn(out);
		for (Medium medium : borrowedMedia) {
			out.println(medium.getSignature());
		}
		out.println();
	}

	public void readFromReader(PersistencyLayer pl, BufferedReader in)
			throws IOException {
		cprNumber = in.readLine();
		name = in.readLine();
		email = in.readLine();
		address = Address.readFrom(in);
		borrowedMedia = new ArrayList<Medium>();
		String signature;
		signature = in.readLine();
		while (!signature.isEmpty()) {
			borrowedMedia.add(pl.readMedium(signature));
			signature = in.readLine();
		}
	}

	public void setName(String string) {
		name = string;
	}

	public String getKey() {
		return getCprNumber();
	}
}
