package dtu.library.persistency;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import dtu.library.app.Address;
import dtu.library.app.Book;
import dtu.library.app.Cd;
import dtu.library.app.LibraryApp;
import dtu.library.app.Medium;
import dtu.library.app.User;

public class TestDatabaseUpdate {
	LibraryApp libApp = new LibraryApp();
	User user;
	Book b;
	Cd c;

	@Before
	public void setUp() throws Exception {
		libApp = new LibraryApp();
		PersistencyLayer.clearDatabase();
		libApp.adminLogin("adminadmin");

		Address address = new Address("Kongevejen", 2120, "København");
		user = new User("cpr-number", "Some Name", "a@b.dk", address);

		libApp.register(user);

		b = new Book("b01", "some book title", "some book author");
		c = new Cd("c01", "some cd title", "some cd author");

		libApp.addMedium(b);
		libApp.addMedium(c);

	}

	@Test
	public void testAdding() throws Exception {
		
		PersistencyLayer pl = new PersistencyLayer();
		User user1 = pl.readUser(user.getCprNumber());
		Utilities.compareUsers(user, user1);
		Medium b1 = pl.readMedium(b.getSignature());
		Utilities.compareMedium(b, b1);
		Medium c1 = pl.readMedium(c.getSignature());
		Utilities.compareMedium(c, c1);
	}

	@Test
	public void testBorrowing() throws Exception {
		user.borrowMedium(b);
		user.borrowMedium(c);

		PersistencyLayer pl = new PersistencyLayer();

		User user1 = (User) pl.readUser(user.getCprNumber());
		assertEquals(2, user1.getBorrowedMedia().size());
		Utilities.compareUsers(user, user1);
	}

	@Test
	public void testReturningBooks() throws Exception {
		user.borrowMedium(b);
		user.borrowMedium(c);

		PersistencyLayer pl = new PersistencyLayer();

		User user1 = (User) pl.readUser(user.getCprNumber());
		assertEquals(2, user1.getBorrowedMedia().size());
		Utilities.compareUsers(user, user1);

		user.returnMedia(b);

		pl = new PersistencyLayer();

		user1 = (User) pl.readUser(user.getCprNumber());
		assertEquals(1, user1.getBorrowedMedia().size());
		Utilities.compareUsers(user, user1);
	}
	
	public void moreTest() {
		fail("Check if more tests are needed");
	}
}
